/******************************************************************************
 ** File Name:    Sensor_SP0827.c                                         *
 ** Author:         Mormo_Hui                                             	  *
 ** Date:		    2011/06/14                                            *
 ** Platform:	    SP6610&6620
 ** Copyright:    Spreadtrum All Rights Reserved.        *
 ** Description:   implementation of digital camera register interface       *
 ******************************************************************************

 ******************************************************************************
 **                        Edit History                                       *
 ** ------------------------------------------------------------------------- *
 ** DATE             NAME             DESCRIPTION                                 *
 ** 2011/06/14   Mormo	     Fristly Released                               *
 ******************************************************************************/
#ifndef _SP0827_C_
#define _SP0827_C_
#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "i2c_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

#include "sc6600l_reg_isp.h"

/**---------------------------------------------------------------------------*
 ** 						Compiler Flag									  *
 **---------------------------------------------------------------------------*/
#ifdef	 __cplusplus
	extern	 "C" 
	{
#endif
/*lint -save -e785 */
/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0827_I2C_ADDR_W    			0x80
#define SP0827_I2C_ADDR_R    			0x81
#define SP0827_I2C_ACK				0x0
LOCAL BOOLEAN  bl_GC_50Hz_SP0827	= FALSE;

#define  SP0827_DAYLIGHT_MAXGAIN                       0x70
#define  SP0827_NIGHT_MAXGAIN                          0xa0
/**---------------------------------------------------------------------------*
     SP0827 SUPPORT 2 SERIAL MODE:
     SPI, CCIR656_2BIT
 **---------------------------------------------------------------------------*/
//#define SP0827_OUTPUT_MODE_SPI
#define SP0827_OUTPUT_MODE_CCIR656_2BIT

/**---------------------------------------------------------------------------*
 **                     Extern Function Prototypes                             *
 **---------------------------------------------------------------------------*/
extern void Init_Main_Camera_For_Sub(SENSOR_OUTPUT_MODE_E mode);
extern uint32 Set_Main_Camera_Brightness(uint32 level);
extern uint32 Set_Main_Camera_Contrast(uint32 level);
extern uint32 Set_Main_Camera_Image_Effect(uint32 effect_type);
extern uint32 Set_Main_Camera_AWB(uint32 mode);

/**---------------------------------------------------------------------------*
 **                     Local Function Prototypes                             *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0827_Power_On(uint32 power_on);
LOCAL uint32 SP0827_Identify(uint32 param);
LOCAL void   SP0827_WriteReg( uint8  subaddr, uint8 data );
LOCAL uint8  SP0827_ReadReg( uint8  subaddr);
LOCAL void   SP0827_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr );
LOCAL void 	 SP0827_Delayms (uint32 ms);
LOCAL uint32 Set_SP0827_Mirror(uint32 level);
LOCAL uint32 Set_SP0827_Flip(uint32 level);
LOCAL uint32 Set_SP0827_Ev(uint32 level);
LOCAL uint32 Set_SP0827_Anti_Flicker(uint32 mode);
LOCAL uint32 Set_SP0827_Preview_Mode(uint32 preview_mode);
LOCAL uint32 SP0827_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable);
LOCAL uint32 SP0827_Before_Snapshot(uint32 para);
LOCAL uint32 SP0827_After_Snapshot(uint32 para);
LOCAL uint32 Set_SP0827_Video_Mode(uint32 mode);
LOCAL uint32 SP0827_GetPclkTab(uint32 param);
/**---------------------------------------------------------------------------*
 ** 						Local Variables 								 *
 **---------------------------------------------------------------------------*/
__align(4) const SENSOR_REG_T SP0827_YUV_240X320[] = 
{
#ifdef 	SP0827_OUTPUT_MODE_SPI   
 //24M 1bit 8-15fps
  {0x1c , 0x00},
  {0x30 , 0x16},
  {0x27 , 0x38},
  {0x28 , 0x0b},
  {0xE7 , 0x03},
  {0xE7 , 0x00},
  {0x32 , 0x00},
  {0x03 , 0x01},
  {0x04 , 0x90},
  {0x24 , 0x2a},

//analog  for 1bit    
  {0x0f , 0x06}, 
  {0x10 , 0x06},  
  {0x12 , 0x05}, 
  {0x13 , 0x03},
  {0x14 , 0x00},
  {0x15 , 0x02},  
  {0x16 , 0x00},
  {0x17 , 0x00},  
  {0x18 , 0x00}, 
  {0x1a , 0x0f}, 

//AE  24M 8Ƶ for 1bit 15-8fps ae setting 

  {0x06 , 0x0 },
  {0x09 , 0x0 },
  {0x0a , 0x2c},
//base      
  {0x97 , 0x32},
  {0x98 , 0x0 },
//exp       
  {0x70 , 0x58},
  {0x71 , 0x2 },
  {0x72 , 0x32},
  {0x73 , 0x0 },
  {0x78 , 0x32},
  {0x79 , 0x0 },
  {0x7a , 0x1 },
  {0x7b , 0x0 },

  #else   
  //24M 2bit 8-20fps
  {0x1c , 0x00},
//sp0827+sp0829
  {0x30 , 0x06},//PCLK ƵISP 8Ƶ
  {0x27 , 0x38},
  {0x28 , 0x0b},
  {0xE7 , 0x03},
  {0xE7 , 0x00},
  {0x32 , 0x00},
  {0x03 , 0x01},
  {0x04 , 0x90},
  {0x24 , 0x2a},

//analog  for 2bit    
  {0x0f , 0x0c}, 
  {0x10 , 0x0c},  
  {0x12 , 0x0a}, 
  {0x13 , 0x06},
  {0x14 , 0x00},
  {0x15 , 0x04},  
  {0x16 , 0x00},
  {0x17 , 0x01},  
  {0x18 , 0x00}, 
  {0x1a , 0x0f}, 

//AE  24M 4Ƶ for 2bit 20-8fps ae setting 

  {0x06 , 0x0 },
  {0x09 , 0x0 },
  {0x0a , 0xc0},
//base      
  {0x97 , 0x43},
  {0x98 , 0x0 },
//exp       
  {0x70 , 0x24},
  {0x71 , 0x03},
  {0x72 , 0x43},
  {0x73 , 0x00},
  {0x78 , 0x43},
  {0x79 , 0x00},
  {0x7a , 0x01},
  {0x7b , 0x00},
  
  #endif
  
  {0x0e , 0x00},
  //gain       
  {0x74 , 0x70},
  {0x75 , 0x2a},
  {0x7c , 0x50},
  {0x7d , 0x2a},
  {0x80 , 0x50},
  {0x81 , 0x3e},
  {0x82 , 0x36},
  {0x83 , 0x30},
  {0x84 , 0x30},
  {0x85 , 0x2e},
  {0x86 , 0x2e},
  {0x87 , 0x2c},
  {0x88 , 0x2c},
  {0x89 , 0x2c},
  {0x8a , 0x2a},
  {0x8b , 0x2a},
  {0x8c , 0x2a},
//target     
  {0x90 , 0x50},//60     //indoor hold
  {0x93 , 0x4c},//5c	//indoor lock
  {0x94 , 0x44},//54
  {0x8f , 0x40},//50 
  {0x92 , 0x50},//60	//outdoor hold
  {0x95 , 0x4c},//5c	//outdoor lock
  {0x96 , 0x44},//54
  {0x91 , 0x40},//50      

//Status     
  {0x60 , 0x40},//mean_nr_dummy
  {0x61 , 0x50},//mean_dummy_nr
  {0x62 , 0x20},//mean_dummy_low
  {0x63 , 0x35},//mean_low_dummy
  {0x64 , 0x70},//rpc_heq_low
  {0x65 , 0x70},//rpc_heq_dummy
  {0x66 , 0x58},//exp_heq_dummy_8lsm
  {0x67 , 0x02},//exp_heq_dummy_5hsm
  {0x68 , 0x58},//exp_heq_low_8lsm
  {0x69 , 0x02},//exp_heq_low_5hsm

  {0x99 , 0x09},  
  {0x32 , 0x05},
};

LOCAL SENSOR_REG_TAB_INFO_T s_SP0827_resolution_Tab_YUV[]=
{   
    { ADDR_AND_LEN_OF_ARRAY(SP0827_YUV_240X320), 240, 320,  24, SENSOR_IMAGE_FORMAT_YUV422 },
    // YUV422 PREVIEW 1
    { PNULL,                    0,   240,    320,    24   ,   SENSOR_IMAGE_FORMAT_YUV422},
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    

    // YUV422 PREVIEW 2 
     { PNULL,                   0,      0,  0 ,         0,         0     }, 
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      }

};


LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0827_ioctl_func_tab = 
{
// Internal 
    PNULL,
    SP0827_Power_On, 
    PNULL,
    SP0827_Identify,
    PNULL,
    PNULL,
    
    PNULL,
    SP0827_GetPclkTab,

// External
    PNULL, 
    Set_SP0827_Mirror,
    Set_SP0827_Flip,
    
    Set_Main_Camera_Brightness,
    Set_Main_Camera_Contrast,
    PNULL, 
    PNULL, 
    Set_SP0827_Preview_Mode, 
    
    Set_Main_Camera_Image_Effect, 
    PNULL,//SP0829_Before_Snapshot,
    PNULL,//SP0829_After_Snapshot,

    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    Set_Main_Camera_AWB,    
    PNULL,
    PNULL, 
    Set_SP0827_Ev, 
    PNULL, 
    PNULL, 
    PNULL,
    
    PNULL,
    PNULL,
    Set_SP0827_Anti_Flicker,
    Set_SP0827_Video_Mode,   
    PNULL
};

 

/**---------------------------------------------------------------------------*
 ** 						Global Variables								  *
 **---------------------------------------------------------------------------*/
PUBLIC SENSOR_INFO_T g_SP0827_yuv_info =
{
	SP0827_I2C_ADDR_W,				// salve i2c write address
	SP0827_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_N|\
	SENSOR_HW_SIGNAL_HSYNC_N,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											

	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		
	
	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,		// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	
	2,								// count of identify code 
	0x02, 0x9b,						// supply two code to identify sensor.
	0x02, 0x9b,						// for Example: index = 0-> Device id, index = 1 -> version id											
											
	SENSOR_AVDD_2800MV,				// voltage of avdd	
	
	240,							// max width of source image
	320,							// max height of source image
	"SP0827",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0827_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0827_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
	SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	4,                     // skip frame num before preview 
	3,                     // skip frame num before capture		
	0,                     // deci frame num during preview;		
	2,                     // deci frame num during video preview;
	0,                     // threshold enable
	0,                     // threshold mode
	0,                     // threshold start postion	
	0,                     // threshold end postion 
	PNULL,
#ifdef 	SP0827_OUTPUT_MODE_SPI
	SENSOR_OUTPUT_MODE_SPI
#elif defined(SP0827_OUTPUT_MODE_CCIR656_2BIT)
	SENSOR_OUTPUT_MODE_CCIR656_2BIT
#else
#endif
};

/**---------------------------------------------------------------------------*
 ** 							Function  Definitions
 **---------------------------------------------------------------------------*/
LOCAL void SP0827_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		uint8 cmd[2];
		cmd[0]	=	subaddr;
		cmd[1]	=	data;		
		I2C_WriteCmdArr(SP0827_I2C_ADDR_W, cmd, 2, SCI_TRUE);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	SCI_TRACE_LOW("SENSOR: SP0827_WriteReg reg/value(%x,%x) !!", subaddr, data);

}

LOCAL uint8 SP0827_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
		I2C_WriteCmdArr(SP0827_I2C_ADDR_W, &subaddr, 1, SCI_TRUE);
		I2C_ReadCmd(SP0827_I2C_ADDR_R, &value, SCI_TRUE);
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif

	//SCI_TRACE_LOW("SENSOR: SP0827_ReadReg reg/value(%x,%x) !!", subaddr, value);
    
	return value;
}


LOCAL void SP0827_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr )
{
    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

}

LOCAL void SP0827_Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}


/**---------------------------------------------------------------------------*
 **                         Function Definitions                              *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0827_Power_On(uint32 power_on)
{
		SENSOR_AVDD_VAL_E		dvdd_val=g_SP0827_yuv_info.dvdd_val;
		SENSOR_AVDD_VAL_E		avdd_val=g_SP0827_yuv_info.avdd_val;
		SENSOR_AVDD_VAL_E		iovdd_val=g_SP0827_yuv_info.iovdd_val;	
		BOOLEAN 				power_down=g_SP0827_yuv_info.power_down_level;		
		BOOLEAN 				reset_level=g_SP0827_yuv_info.reset_pulse_level;
		uint32				reset_width=g_SP0827_yuv_info.reset_pulse_width;		
		
		if(SCI_TRUE==power_on)
		{
			Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
			SCI_Sleep(10);
			GPIO_SetSensorPwdn(!power_down);
	
			//Open Mclk in default frequency
			Sensor_SetMCLK(24);   
			 SCI_Sleep(50);
			//sp0829 ISP 	
#ifdef SP0827_OUTPUT_MODE_SPI 
			 Init_Main_Camera_For_Sub(SENSOR_OUTPUT_MODE_SPI);
#elif defined(SP0827_OUTPUT_MODE_CCIR656_2BIT)
			 Init_Main_Camera_For_Sub(SENSOR_OUTPUT_MODE_CCIR656_2BIT);

#endif     
	
		}
		else
		{
			GPIO_SetSensorPwdn(power_down);
	
			Sensor_SetMCLK(SENSOR_DISABLE_MCLK);		   
	
			Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);	  
			SCI_Sleep(20);
		}
		
	
		return SCI_SUCCESS;
}



/******************************************************************************/
// Description: sensor probe function
// Author:     benny.zou
// Input:      none
// Output:     result
// Return:     0           successful
//             others      failed
// Note:       this function only to check whether sensor is work, not identify 
//              whitch it is!!
/******************************************************************************/
LOCAL uint32 Set_SP0827_Mirror(uint32 level)
{
	return (uint32)SCI_SUCCESS;
}
	
LOCAL uint32 Set_SP0827_Flip(uint32 level)
{
	return (uint32)SCI_SUCCESS;
}
	
LOCAL uint32 SP0827_Identify(uint32 param)
{
#if 1
		uint8 reg[2]	= {0x02, 0x02};
		//uint8 value[2]  = {0x0c, 0x0c};	//0828
		uint8 value[2]	= {0x9b, 0x9b}; //0829
		uint8 ret		= 0;
		uint32 i;
		uint8 err_cnt = 0;
		uint32 nLoop = 1000;
	
	//	SP0827_DELAY_MS(10);
	
		for(i = 0; i<2; )
		{
			nLoop = 1000;
			//SP0827_WriteReg(0xfd,0x00);  
			ret = SP0827_ReadReg(reg[i]);	 
			
			SCI_TRACE_LOW( "SP0827 read ret is %X",ret );
			
			if( ret != value[i])
			{
				err_cnt++;
				if(err_cnt>3)
				{
					SCI_TRACE_LOW( "SP0827 Fail" );
					return 0xFF;
				}
				else
				{
					//Masked by frank.yang,SCI_Sleep() will cause a  Assert when called in boot precedure
					//SCI_Sleep(10);
					while(nLoop--);
					continue;
				}
			}
			err_cnt = 0;
			i++;
		}
	
		 SCI_TRACE_LOW( "SP0827 succ" );
#endif

	return (uint32)SCI_SUCCESS;
	
}


LOCAL SENSOR_TRIM_T s_SP0827_Pclk_Tab[]=
{	
	// COMMON INIT
	{0, 0, 0, 0,  6},
	
	// YUV422 PREVIEW 1	
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	
	// YUV422 PREVIEW 2 
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0}
};

LOCAL uint32 SP0827_GetPclkTab(uint32 param)
{
    return (uint32)s_SP0827_Pclk_Tab;
}

__align(4) const SENSOR_REG_T SP0827_ev_tab[][9]=
{   
    {{0x8f, 0x40-0x18},{0x90, 0x50-0x18},{0x91, 0x40-0x18},{0x92,0x50-0x18},{0x93, 0x4c-0x18},{0x94, 0x44-0x18},{0x95, 0x4c-0x18},{0x96, 0x44-0x18},{0xff, 0xff}},
    {{0x8f, 0x40-0x10},{0x90, 0x50-0x10},{0x91, 0x40-0x10},{0x92,0x50-0x10},{0x93, 0x4c-0x10},{0x94, 0x44-0x10},{0x95, 0x4c-0x10},{0x96, 0x44-0x10},{0xff, 0xff}},
    {{0x8f, 0x40-0x08},{0x90, 0x50-0x08},{0x91, 0x40-0x08},{0x92,0x50-0x08},{0x93, 0x4c-0x08},{0x94, 0x44-0x08},{0x95, 0x4c-0x08},{0x96, 0x44-0x08},{0xff, 0xff}},
    {{0x8f, 0x40-0x00},{0x90, 0x50-0x00},{0x91, 0x40-0x00},{0x92,0x50-0x00},{0x93, 0x4c-0x00},{0x94, 0x44-0x00},{0x95, 0x4c-0x00},{0x96, 0x44-0x00},{0xff, 0xff}},
    {{0x8f, 0x40+0x08},{0x90, 0x50+0x08},{0x91, 0x40+0x08},{0x92,0x50+0x08},{0x93, 0x4c+0x08},{0x94, 0x44+0x08},{0x95, 0x4c+0x08},{0x96, 0x44+0x08},{0xff, 0xff}},
    {{0x8f, 0x40+0x10},{0x90, 0x50+0x10},{0x91, 0x40+0x10},{0x92,0x50+0x10},{0x93, 0x4c+0x10},{0x94, 0x44+0x10},{0x95, 0x4c+0x10},{0x96, 0x44+0x10},{0xff, 0xff}},
    {{0x8f, 0x40+0x18},{0x90, 0x50+0x18},{0x91, 0x40+0x18},{0x92,0x50+0x18},{0x93, 0x4c+0x18},{0x94, 0x44+0x18},{0x95, 0x4c+0x18},{0x96, 0x44+0x18},{0xff, 0xff}},  
};

LOCAL uint32 Set_SP0827_Ev(uint32 level)
{
  
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0827_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    SCI_ASSERT(level < 7);
 
    SP0827_Write_Group_Regs(sensor_reg_ptr );

    SCI_TRACE_LOW("set_SP0827_ev: level = %d", level);

    return 0;
}

/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/
LOCAL uint32 Set_SP0827_Anti_Flicker(uint32 mode)
{ 
	switch(mode)
	{
	case DCAMERA_FLICKER_50HZ:
	
		bl_GC_50Hz_SP0827 = TRUE;
		SCI_TRACE_LOW( " set_SP0827_anti_flicker  50hz \r\n" );

         break;       
	case DCAMERA_FLICKER_60HZ:
	
	    bl_GC_50Hz_SP0827 = FALSE;
	       SCI_TRACE_LOW( " set_SP0827_anti_flicker  60hz \r\n" );
			 
	break;

	    default:
	    break;
    }
      
    SCI_TRACE_LOW("set_SP0827_anti_flicker-mode=%d",mode);
    OS_TickDelay(100); 
	
    return 0;
}

LOCAL uint32 Set_SP0827_Preview_Mode(uint32 preview_mode)
{
	switch (preview_mode)
	{
	  case DCAMERA_ENVIRONMENT_NORMAL: 
	     {
	       if( bl_GC_50Hz_SP0827 == TRUE)
	       {
		SCI_TRACE_LOW("normal mode 50hz\r\n");
		#ifdef 	SP0827_OUTPUT_MODE_SPI
		//capture preview daylight 24M 8Ƶ 50HZ for 1bit 15-8fps 
	  			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0827_WriteReg(0x97,0x32);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x58);//exp			 
	        SP0827_WriteReg(0x71,0x02);			 
	        SP0827_WriteReg(0x72,0x32);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x32);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x58);
		SP0827_WriteReg(0x67,0x02);
		SP0827_WriteReg(0x68,0x58);
		SP0827_WriteReg(0x69,0x02);
               #else
               //capture preview daylight 24M 4Ƶ 50HZ for 2bit 20-8fps 
               			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0xc0);  			 
	         				   
	        SP0827_WriteReg(0x97,0x43);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x24);//exp			 
	        SP0827_WriteReg(0x71,0x03);			 
	        SP0827_WriteReg(0x72,0x43);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x43);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x24);
		SP0827_WriteReg(0x67,0x03);
		SP0827_WriteReg(0x68,0x24);
		SP0827_WriteReg(0x69,0x03);  
		   				
		#endif 
		}       
	   else	
		 {
		SCI_TRACE_LOW("normal mode 60hz\r\n");
		#ifdef 	SP0827_OUTPUT_MODE_SPI                          					
		//capture preview daylight 24M 8Ƶ  60HZ for 1bit 15-8fps    				 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0x40);  			 
	         				   
	        SP0827_WriteReg(0x97,0x27);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x49);//exp			 
	        SP0827_WriteReg(0x71,0x02);			 
	        SP0827_WriteReg(0x72,0x27);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x27);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x49);
		SP0827_WriteReg(0x67,0x02);
		SP0827_WriteReg(0x68,0x49);
		SP0827_WriteReg(0x69,0x02);	
               #else 
                //capture preview daylight 24M 4Ƶ 60HZ for 2bit 20-8fps   		
                                                        					 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0xbe);  			 
	         				   
	        SP0827_WriteReg(0x97,0x38);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x48);//exp			 
	        SP0827_WriteReg(0x71,0x03);			 
	        SP0827_WriteReg(0x72,0x38);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x38);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x48);
		SP0827_WriteReg(0x67,0x03);
		SP0827_WriteReg(0x68,0x48);
		SP0827_WriteReg(0x69,0x03);           		
               #endif
               }
	       break;
	    }
		case DCAMERA_ENVIRONMENT_NIGHT:
		{
		if( bl_GC_50Hz_SP0827 == TRUE)
		{
		SCI_TRACE_LOW("night mode 50hz\r\n");
		#ifdef 	SP0827_OUTPUT_MODE_SPI
		   //capture preview night 24M 8Ƶ 50HZ for 1bit 15-5fps 			 			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0827_WriteReg(0x97,0x32);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0xe8);//exp			 
	        SP0827_WriteReg(0x71,0x03);			 
	        SP0827_WriteReg(0x72,0x32);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x32);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);
                
		SP0827_WriteReg(0x74,SP0827_NIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_NIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0xe8);
		SP0827_WriteReg(0x67,0x03);
		SP0827_WriteReg(0x68,0xe8);
		SP0827_WriteReg(0x69,0x03);	
               #else
                  //capture preview night 24M 4Ƶ 50HZ for 2bit 20-5fps 	 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0xc0);  			 
	         				   
	        SP0827_WriteReg(0x97,0x43);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x3c);//exp			 
	        SP0827_WriteReg(0x71,0x05);			 
	        SP0827_WriteReg(0x72,0x43);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x43);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);
                
		SP0827_WriteReg(0x74,SP0827_NIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_NIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x3c);
		SP0827_WriteReg(0x67,0x05);
		SP0827_WriteReg(0x68,0x3c);
		SP0827_WriteReg(0x69,0x05);  
		    				
	      #endif 			
		}
	  else
		{
		SCI_TRACE_LOW("night mode 60hz\r\n");
		#ifdef 	SP0827_OUTPUT_MODE_SPI
		   //capture preview night 24M 8Ƶ 60HZ for 1bit 15-5fps         				 			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0x40);  			 
	         				   
	        SP0827_WriteReg(0x97,0x27);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0xa8);//exp			 
	        SP0827_WriteReg(0x71,0x03);			 
	        SP0827_WriteReg(0x72,0x27);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x27);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_NIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_NIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0xa8);
		SP0827_WriteReg(0x67,0x03);
		SP0827_WriteReg(0x68,0xa8);
		SP0827_WriteReg(0x69,0x03);	
               #else 
                //capture preview night 24M 4Ƶ 60HZ for 2bit 20-5fps    		
                                                        				 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0xbe);  			 
	         				   
	        SP0827_WriteReg(0x97,0x38);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x40);//exp			 
	        SP0827_WriteReg(0x71,0x05);			 
	        SP0827_WriteReg(0x72,0x38);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x38);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_NIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_NIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_NIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x40);
		SP0827_WriteReg(0x67,0x05);
		SP0827_WriteReg(0x68,0x40);
		SP0827_WriteReg(0x69,0x05);
		
		#endif
		}               
	        break;  
		}		
		default:
		{
			break;
		}
		
	}       
      SCI_TRACE_LOW("set_SP0827_preview_mode: level = %d", preview_mode);
      return 0;
}

LOCAL uint32 SP0827_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable)
{
	uint16 ae_value = 0 , awb_value=0;

	awb_value = SP0827_ReadReg(0x32);

	if(0x01==ae_enable)
	{
		ae_value |= 0x01;
	}
	else if(0x00==ae_enable)
	{
		ae_value &=0xfe;
	}

	if(0x01==awb_enable)
	{
		awb_value |=0x08;
	}
	else if(0x00==awb_enable)
	{
		awb_value &= 0xf7;
	}	
	
	SP0827_WriteReg(0x32, ae_value);
	SP0827_WriteReg(0x32, awb_value);

       SCI_TRACE_LOW("SENSOR: set_ae_awb_enable: ae_enable = %d  awb_enable = %d", ae_enable ,awb_enable);
	return 0;
}


LOCAL uint32 SP0827_Before_Snapshot(uint32 para)
{
	uint8 reg_val = 0;
	uint16 exposal_time=0x00;

	SP0827_AE_AWB_Enable(0x32,0x00);   // close aec awb

	reg_val = SP0827_ReadReg(0x04);
	exposal_time=reg_val&0x00ff;
	reg_val = SP0827_ReadReg(0x03);
	exposal_time|=((reg_val&0x00ff)<<0x08);

	if(exposal_time<1)
	{
		exposal_time=1;
	}

	reg_val=exposal_time&0x00ff;
	SP0827_WriteReg(0x04, reg_val);
	reg_val=(exposal_time&0xff00)>>8;
	SP0827_WriteReg(0x03, reg_val);

	SP0827_Delayms(400);	
	
	SCI_TRACE_LOW("SENSOR_SP0827: Before Snapshot");

	return 0;
}



LOCAL uint32 SP0827_After_Snapshot(uint32 para)
{
	SP0827_AE_AWB_Enable(0x32,0x0d);   // Open aec awb
	
	SP0827_Delayms(400);
	
	SCI_TRACE_LOW("SENSOR_SP0827: After Snapshot");
	//SP0827_Power_On(SCI_FALSE);

	return 0;
}


 
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/   
LOCAL uint32 Set_SP0827_Video_Mode(uint32 mode)
{
    
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

    if(1==mode ||2==mode )
    {
    	if(bl_GC_50Hz_SP0827 == TRUE)  // is 50hz in video
    	{
#ifdef 	SP0827_OUTPUT_MODE_SPI
		//AE 24M 8Ƶ fix 14fps			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x00);			 
	        SP0827_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0827_WriteReg(0x97,0x32);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x5e);//exp			 
	        SP0827_WriteReg(0x71,0x01);			 
	        SP0827_WriteReg(0x72,0x32);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x32);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);
			
		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x5e);
		SP0827_WriteReg(0x67,0x01);
		SP0827_WriteReg(0x68,0x5e);
		SP0827_WriteReg(0x69,0x01);	
  #else
  //AE 24M 4Ƶ for 2bit fix 14fps			 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x01);			 
	        SP0827_WriteReg(0x0a,0x7e);  			 
	         				   
	        SP0827_WriteReg(0x97,0x2f);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x49);//exp			 
	        SP0827_WriteReg(0x71,0x01);			 
	        SP0827_WriteReg(0x72,0x2f);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x2f);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x49);
		SP0827_WriteReg(0x67,0x01);
		SP0827_WriteReg(0x68,0x49);
		SP0827_WriteReg(0x69,0x01);   
	                        
#endif

	}
	else  // is 60hz in video mode
	{
#ifdef 	SP0827_OUTPUT_MODE_SPI                          					
		//AE 24M 8Ƶ for 1bit fix 14fps         					
	        SP0827_WriteReg(0x06,0x00);				
	        SP0827_WriteReg(0x09,0x00);				
	        SP0827_WriteReg(0x0a,0x40);  				
	         			  	        		
	        SP0827_WriteReg(0x97,0x27);//base			
	        SP0827_WriteReg(0x98,0x00);				
	                        	  	  			
	        SP0827_WriteReg(0x70,0x38);//exp			
	        SP0827_WriteReg(0x71,0x01);				
	        SP0827_WriteReg(0x72,0x27);				
	        SP0827_WriteReg(0x73,0x00);				
	        SP0827_WriteReg(0x78,0x27);				
	        SP0827_WriteReg(0x79,0x00);				
	        SP0827_WriteReg(0x7a,0x01);				
	        SP0827_WriteReg(0x7b,0x00);    
			            
		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x38);
		SP0827_WriteReg(0x67,0x01);
		SP0827_WriteReg(0x68,0x38);
		SP0827_WriteReg(0x69,0x01); 	
  #else                                         
  //AE 24M 4Ƶ for 2bit fix 14fps       		
                                                            		 
	        SP0827_WriteReg(0x06,0x00);			 
	        SP0827_WriteReg(0x09,0x01);			 
	        SP0827_WriteReg(0x0a,0x81);  			 
	         				   
	        SP0827_WriteReg(0x97,0x27);//base			 
	        SP0827_WriteReg(0x98,0x00);			 
	                        		  	   
	        SP0827_WriteReg(0x70,0x38);//exp			 
	        SP0827_WriteReg(0x71,0x01);			 
	        SP0827_WriteReg(0x72,0x27);			 
	        SP0827_WriteReg(0x73,0x00);			 
	        SP0827_WriteReg(0x78,0x27);			 
	        SP0827_WriteReg(0x79,0x00);			 
	        SP0827_WriteReg(0x7a,0x01);			 
	        SP0827_WriteReg(0x7b,0x00);

		SP0827_WriteReg(0x74,SP0827_DAYLIGHT_MAXGAIN);	
		SP0827_WriteReg(0x7c,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x64,SP0827_DAYLIGHT_MAXGAIN);
		SP0827_WriteReg(0x65,SP0827_DAYLIGHT_MAXGAIN);
		
		SP0827_WriteReg(0x66,0x38);
		SP0827_WriteReg(0x67,0x01);
		SP0827_WriteReg(0x68,0x38);
		SP0827_WriteReg(0x69,0x01);                  		
          
#endif				 
	}
    SCI_TRACE_LOW("set_SP0827_video_mode=%d",mode);
    }	
	
    return 0;
}

void Sub_Camera_Disable()
{
	SP0827_WriteReg(0x0e, 0xff);
	SP0827_WriteReg(0x1c, 0x03);
	SP0827_WriteReg(0xe7, 0x03);
	SP0827_WriteReg(0xe7, 0x00);
}

void ATC27_WriteReg(uint8 add, uint8 val)
{
	SP0827_WriteReg(add, val);
}
uint8 ATC27_ReadReg(uint8 add)
{
	return SP0827_ReadReg(add);
}
/*lint -restore */
/**---------------------------------------------------------------------------------------------*
 **                         Compiler Flag                             					        *
 **---------------------------------------------------------------------------------------------*/
#ifdef   __cplusplus
    }
#endif

#endif
