/******************************************************************************
 ** File Name:    Sensor_SP0829.c                                         *
 ** Author:         Mormo_Hui                                             	  *
 ** Date:		    2011/06/14                                            *
 ** Platform:	    SP6610&6620
 ** Copyright:    Spreadtrum All Rights Reserved.        *
 ** Description:   implementation of digital camera register interface       *
 ******************************************************************************

 ******************************************************************************
 **                        Edit History                                       *
 ** ------------------------------------------------------------------------- *
 ** DATE             NAME             DESCRIPTION                                 *
 ** 2011/06/14   Mormo	     Fristly Released                               *
 ******************************************************************************/
#ifndef _SP0829_C_
#define _SP0829_C_
#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "i2c_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

#include "sc6600l_reg_isp.h"

/**---------------------------------------------------------------------------*
 ** 						Compiler Flag									  *
 **---------------------------------------------------------------------------*/
#ifdef	 __cplusplus
	extern	 "C" 
	{
#endif
/*lint -save -e785 */
/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0829_I2C_ADDR_W    			0x86
#define SP0829_I2C_ADDR_R    			0x87
#define SP0829_I2C_ACK					0x0
LOCAL BOOLEAN  	bl_GC_50Hz_SP0829		= FALSE;
LOCAL BOOLEAN g_isMain = TRUE;

#define  SP0829_DAYLIGHT_MAXGAIN                       0x70
#define  SP0829_NIGHT_MAXGAIN                          0xa0

/**---------------------------------------------------------------------------*
     SP0829 SUPPORT 3 SERIAL MODE:
     SPI, CCIR656_2BIT, CCIR656_4BIT
 **---------------------------------------------------------------------------*/
//#define SP0829_OUTPUT_MODE_SPI
#define SP0829_OUTPUT_MODE_CCIR656_2BIT
//#define SP0829_OUTPUT_MODE_CCIR656_4BIT

/**---------------------------------------------------------------------------*
     IF SUPPORT SUB CAMERA WITH SP0827 DEFINE THIS
 **---------------------------------------------------------------------------*/
#define SUB_CAMERA_SUPPORT

/**---------------------------------------------------------------------------*
 **                     Local Function Prototypes                             *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0829_Power_On(uint32 power_on);
LOCAL uint32 SP0829_Identify(uint32 param);
LOCAL void   SP0829_WriteReg( uint8  subaddr, uint8 data );
LOCAL uint8  SP0829_ReadReg( uint8  subaddr);
LOCAL void   SP0829_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr );
LOCAL void 	 SP0829_Delayms (uint32 ms);
LOCAL uint32 Set_SP0829_Mirror(uint32 level);
LOCAL uint32 Set_SP0829_Flip(uint32 level);
LOCAL uint32 Set_SP0829_Brightness(uint32 level);
LOCAL uint32 Set_SP0829_Contrast(uint32 level);
LOCAL uint32 Set_SP0829_Image_Effect(uint32 effect_type);
LOCAL uint32 Set_SP0829_Ev(uint32 level);
LOCAL uint32 Set_SP0829_Anti_Flicker(uint32 mode);
LOCAL uint32 Set_SP0829_Preview_Mode(uint32 preview_mode);
LOCAL uint32 Set_SP0829_AWB(uint32 mode);
LOCAL uint32 SP0829_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable);
LOCAL uint32 SP0829_Before_Snapshot(uint32 para);
LOCAL uint32 SP0829_After_Snapshot(uint32 para);
LOCAL uint32 Set_SP0829_Video_Mode(uint32 mode);
LOCAL uint32 SP0829_GetPclkTab(uint32 param);
/**---------------------------------------------------------------------------*
 ** 						Local Variables 								 *
 **---------------------------------------------------------------------------*/
__align(4) const SENSOR_REG_T SP0829_YUV_240X320[] = 
{
#ifdef 	SP0829_OUTPUT_MODE_SPI
	
	//24M 1bit 8-15fps
	{0xfd,0x00},
	{0x0e,0x00},
	{0x1c,0x00},			
	//spi		  
	{0x2e,0x00},			
	{0x2c,0x0b},			
	{0x2b,0x0b},			
	{0x2d,0x0b},			
	{0x30,0x06},			
	{0x31,0x00},			
	{0x35,0x40},			
	//analog  for 1bit		  
	{0x0f,0x06},			
	{0x10,0x06},			
	{0x12,0x05},			
	{0x13,0x07},			
	{0x15,0x04},			
	{0x16,0x00},			

	{0x1a,0xa8},			
	{0x27,0x88},			
	{0x28,0x03},			

	{0x32,0x00},			
	{0x29,0x53},	

	//AE 24M 8Ƶ for 1bit 15-8fps
	{0x05,0x00},			 
	{0x06,0x00},			 
	{0x09,0x00},			 
	{0x0a,0x2c},			 
	//base				   
	{0x9b,0x32},			 
	{0x9c,0x00},			 
	//exp				   
	{0xa2,0x58},			 
	{0xa3,0x02},			 
	{0xa4,0x32},			 
	{0xa5,0x00},			 
	{0xa8,0x32},			 
	{0xa9,0x00},			 
	{0xaa,0x01},			 
	{0xab,0x00},			 
	
#elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)
	
	//24M 2bit 8-20fps              
	{0xfd,0x00}, 
	{0x0e,0x00},                   
	{0x1c,0x00},			              
	//spi		                        
	{0x2e,0x11},			              
	{0x2c,0x08},			              
	{0x2b,0x0a},			              
	{0x2d,0x40},			              
	{0x30,0x04},			              
	{0x31,0x00},			              
	{0x35,0x40},			              
	//analog  for 2bit		          
	{0x0f,0x0c},			              
	{0x10,0x0c},			              
	{0x12,0x0a},			              
	{0x13,0x0e},			              
	{0x15,0x08},			              
	{0x16,0x00},			              
                                  
	{0x1a,0xa8},			              
	{0x27,0x88},			              
	{0x28,0x03},			              
                                  
	{0x32,0x00},			              
	{0x29,0x53},			        
                                  
	//AE 24M 4Ƶ for 2bit 20-8fps 
	{0x05,0x00},			              
	{0x06,0x29},			              
	{0x09,0x00},			              
	{0x0a,0x90},			              
	//base				                  
	{0x9b,0x4b},			              
	{0x9c,0x00},			              
	//exp				                    
	{0xa2,0x84},			              
	{0xa3,0x03},			              
	{0xa4,0x4b},			              
	{0xa5,0x00},			              
	{0xa8,0x4b},			              
	{0xa9,0x00},			              
	{0xaa,0x01},			              
	{0xab,0x00},			              
	     
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)
	
	//24M 4bit 8-20fps              
	{0xfd,0x00}, 
	{0x0e,0x00},                   
	{0x1c,0x00},			              
	//spi		                        
	{0x2e,0x12},			              
	{0x2c,0x08},			              
	{0x2b,0x00},			              
	{0x2d,0x00},			              
	{0x30,0x02},			              
	{0x31,0x00},			              
	{0x35,0x40},			              
	//analog  for 4bit		          
	{0x0f,0x18},			              
	{0x10,0x18},			              
	{0x12,0x14},			              
	{0x13,0x1c},			              
	{0x15,0x0c},			              
	{0x16,0x00},			              
                                  
	{0x1a,0xa8},			              
	{0x27,0x88},			              
	{0x28,0x03},			              
                                  
	{0x32,0x00},			              
	{0x29,0x53},			              
	
	//AE 24M 2Ƶ for 4bit 20-8fps 
	{0x05,0x01},			              
	{0x06,0xa0},			              
	{0x09,0x00},			              
	{0x0a,0x90},			              
	//base				                  
	{0x9b,0x96},			              
	{0x9c,0x00},			              
	//exp				                    
	{0xa2,0x08},			              
	{0xa3,0x07},			              
	{0xa4,0x96},			              
	{0xa5,0x00},			              
	{0xa8,0x96},			              
	{0xa9,0x00},			              
	{0xaa,0x01},			              
	{0xab,0x00},	
 #endif	
 
	//pregain	
	{0x4b,0x0c},
	{0x4c,0x0c},
	{0x4d,0x0c},
	{0x4e,0x0c},		              
	//gain				   
	{0xa6,0x70},		  
	{0xa7,0x2a},			
	{0xac,0x50},			
	{0xad,0x2a},	
	{0x8a,0x50},
	{0x8b,0x3e},
	{0x8c,0x36},
	{0x8d,0x30}, 
	{0x8e,0x30},
	{0x8f,0x2e},
	{0x90,0x2e},
	{0x91,0x2c},
	{0x92,0x2c},
	{0x93,0x2c},
	{0x94,0x2a},
	{0x95,0x2a},
	{0x96,0x2a},			
			
	//AWB	    
	{0xc5,0xc4},
	{0xc6,0x9f},
	{0xbe,0x20},
	{0xb3,0x7e},
	{0xb5,0x10},
	{0xd4,0xff},
	{0xba,0x2d},
	{0xd7,0x90},
	{0xd8,0xa0},
	{0xd9,0xff},
	{0xdd,0xa6},
	{0xe1,0xc7},
	{0xe3,0xd0},
	{0xd5,0xe0},
                
	                             
        //smooth	       
	{0x42,0x54},    	                         
	{0x46,0x0a},    	                         
	                	                         
	//sharpen		                         
	{0x50,0x40},    	                         
	{0x51,0x10},    	                         
	{0x52,0xe0},    	                         
	{0x55,0x10}, 	                         
	{0x56,0x30},    	                         
	{0x57,0x20},  
	//color 
        {0x79,0xb1},
        {0x7a,0xcf},
        {0x7b,0x00},
        {0x7c,0x19},
        {0x7d,0x70},
        {0x7e,0xf7},
        {0x7f,0x18},
        {0x80,0xc1},
        {0x81,0xa6},
        {0x82,0x0c},
        {0x83,0x30},
        {0x84,0x0c},
        
       /*//color ԭԺ
        {0x79,0x96},
        {0x7a,0xf1},
        {0x7b,0xfa},
        {0x7c,0xe5},
        {0x7d,0xc6},
        {0x7e,0xd6},
        {0x7f,0xff},
        {0x80,0x96},
        {0x81,0xeb},
        {0x82,0x3c},
        {0x83,0x33},
        {0x84,0x0f},*/  
        
        //sat    
        {0x4f,0x60},
        {0x58,0xff},
        {0x59,0xff},

        //heq
         {0x5b,0x80},   
         {0x5c,0xa0},  			              
                                  
	//auto lum lowlight 		        
	{0x86,0x30},			              
	{0x87,0x1f},			              
	{0x88,0x30},			              
	{0x89,0x45},						        
                                  
		              
	//target

	{0x98,0x88},//0x80,		   
	{0x9e,0x84},//0x7c,		   
	{0x9f,0x7c},//0x74,		   
	{0x97,0x78},//0x70,		   
	{0x9a,0x84},//0x80,		   
	{0xa0,0x80},//0x7c,		   
	{0xa1,0x78},//0x74,		   
	{0x99,0x74},//0x70,
     	//gama
        {0x65,0x0 },
        {0x66,0xa },
        {0x67,0x13},
        {0x68,0x25},
        {0x69,0x43},
        {0x6a,0x5d},
        {0x6b,0x74},
        {0x6c,0x88},
        {0x6d,0x9a},
        {0x6e,0xa9},
        {0x6f,0xb5},
        {0x70,0xc0},
        {0x71,0xca},
        {0x72,0xd4},
        {0x73,0xdd},
        {0x74,0xe6},
        {0x75,0xef},
        {0x76,0xf7},
        {0x77,0xff},
        
	{0x9d,0x09},		                
                                  
	{0x33,0x0f},		                
	{0x34,0x06},		                
	{0x32,0x0d},                    
	{0xff,0xff},                    
                                  
};


LOCAL SENSOR_REG_TAB_INFO_T s_SP0829_resolution_Tab_YUV[]=
{   
    { ADDR_AND_LEN_OF_ARRAY(SP0829_YUV_240X320), 240, 320,  24, SENSOR_IMAGE_FORMAT_YUV422 },
    // YUV422 PREVIEW 1
    { PNULL,                    0,   240,    320,    24   ,   SENSOR_IMAGE_FORMAT_YUV422},
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    

    // YUV422 PREVIEW 2 
     { PNULL,                   0,      0,  0 ,         0,         0     }, 
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      }

};


LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0829_ioctl_func_tab = 
{
// Internal 
    PNULL,
    SP0829_Power_On, 
    PNULL,
    SP0829_Identify,
    PNULL,
    PNULL,
    
    PNULL,
    SP0829_GetPclkTab,

// External
    PNULL, 
    Set_SP0829_Mirror,
    Set_SP0829_Flip,
    
    Set_SP0829_Brightness,
    Set_SP0829_Contrast,
    PNULL, 
    PNULL, 
    Set_SP0829_Preview_Mode, 
    
    Set_SP0829_Image_Effect, 
    PNULL,//SP0829_Before_Snapshot,
    PNULL,//SP0829_After_Snapshot,

    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    Set_SP0829_AWB,    
    PNULL,
    PNULL, 
    Set_SP0829_Ev, 
    PNULL, 
    PNULL, 
    PNULL,
    
    PNULL,
    PNULL,
    Set_SP0829_Anti_Flicker,
    Set_SP0829_Video_Mode,   
    PNULL
};

 

/**---------------------------------------------------------------------------*
 ** 						Global Variables								  *
 **---------------------------------------------------------------------------*/
PUBLIC SENSOR_INFO_T g_SP0829_yuv_info =
{
	SP0829_I2C_ADDR_W,				// salve i2c write address
	SP0829_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_N|\
	SENSOR_HW_SIGNAL_HSYNC_N,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											

	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		
	
	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,		// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	
	2,								// count of identify code 
	0x02, 0x9a,						// supply two code to identify sensor.
	0x02, 0x9a,						// for Example: index = 0-> Device id, index = 1 -> version id											
											
	SENSOR_AVDD_2800MV,				// voltage of avdd	
	
	240,							// max width of source image
	320,							// max height of source image
	"SP0829",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0829_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0829_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
	SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	4,                     // skip frame num before preview 
	3,                     // skip frame num before capture		
	0,                     // deci frame num during preview;		
	2,                     // deci frame num during video preview;
	0,                     // threshold enable
	0,                     // threshold mode
	0,                     // threshold start postion	
	0,                     // threshold end postion 
	PNULL,
#ifdef 	SP0829_OUTPUT_MODE_SPI
	SENSOR_OUTPUT_MODE_SPI
#elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)
	SENSOR_OUTPUT_MODE_CCIR656_2BIT
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)
	SENSOR_OUTPUT_MODE_CCIR656_4BIT
#else
	SENSOR_OUTPUT_MODE_CCIR601_8BIT  
#endif
};

/**---------------------------------------------------------------------------*
 ** 							Function  Definitions
 **---------------------------------------------------------------------------*/
LOCAL void SP0829_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		uint8 cmd[2];
		cmd[0]	=	subaddr;
		cmd[1]	=	data;		
		I2C_WriteCmdArr(SP0829_I2C_ADDR_W, cmd, 2, SCI_TRUE);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	//SCI_TRACE_LOW("SENSOR: SP0829_WriteReg reg/value(%x,%x) !!", subaddr, data);

}

LOCAL uint8 SP0829_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
		I2C_WriteCmdArr(SP0829_I2C_ADDR_W, &subaddr, 1, SCI_TRUE);
		I2C_ReadCmd(SP0829_I2C_ADDR_R, &value, SCI_TRUE);
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif

	//SCI_TRACE_LOW("SENSOR: SP0829_ReadReg reg/value(%x,%x) !!", subaddr, value);
    
	return value;
}


LOCAL void SP0829_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr )
{
    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        //Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		SP0829_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

}

LOCAL void SP0829_Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}


/**---------------------------------------------------------------------------*
 **                         Function Definitions                              *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0829_Power_On(uint32 power_on)
{
		SENSOR_AVDD_VAL_E		dvdd_val=g_SP0829_yuv_info.dvdd_val;
		SENSOR_AVDD_VAL_E		avdd_val=g_SP0829_yuv_info.avdd_val;
		SENSOR_AVDD_VAL_E		iovdd_val=g_SP0829_yuv_info.iovdd_val;	
		BOOLEAN 				power_down=g_SP0829_yuv_info.power_down_level;		
		BOOLEAN 				reset_level=g_SP0829_yuv_info.reset_pulse_level;
		uint32				reset_width=g_SP0829_yuv_info.reset_pulse_width;		
		
		if(SCI_TRUE==power_on)
		{
			Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
				SCI_Sleep(10);
#ifdef DOUBLE_CAMERA_SUPPORT
			if (Sensor_GetFrontCameraState())
			{
				GPIO_SetFrontSensorPwdn(!power_down);
				GPIO_SetSensorPwdn(power_down);
			}
			else
			{
				GPIO_SetFrontSensorPwdn(power_down);
				GPIO_SetSensorPwdn(!power_down);
			}
#else      
				GPIO_SetSensorPwdn(!power_down);
#endif
	
			//Open Mclk in default frequency
			Sensor_SetMCLK(24);  

                    SCI_Sleep(50);
#ifdef SUB_CAMERA_SUPPORT
			//when use back camera, it is better to disable front camera
	        Sub_Camera_Disable();
#endif
	
			//No reset pin on SP0829
			/*
				SCI_Sleep(20);
				Sensor_SetResetLevel(reset_level);
				SCI_Sleep(reset_width);
				Sensor_SetResetLevel(!reset_level);
				SCI_Sleep(100);
				*/
		}
		else
		{
#ifdef DOUBLE_CAMERA_SUPPORT
			GPIO_SetFrontSensorPwdn(power_down);
			GPIO_SetSensorPwdn(power_down);
#else	
			GPIO_SetSensorPwdn(power_down);
#endif
	
			Sensor_SetMCLK(SENSOR_DISABLE_MCLK);		   
	
			Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);	  
			SCI_Sleep(20);
		}
		
	
		return SCI_SUCCESS;
}



/******************************************************************************/
// Description: sensor probe function
// Author:     benny.zou
// Input:      none
// Output:     result
// Return:     0           successful
//             others      failed
// Note:       this function only to check whether sensor is work, not identify 
//              whitch it is!!
/******************************************************************************/
LOCAL uint32 Set_SP0829_Mirror(uint32 level)
{
	return (uint32)SCI_SUCCESS;
}
	
LOCAL uint32 Set_SP0829_Flip(uint32 level)
{
	return (uint32)SCI_SUCCESS;
}

LOCAL uint32 SP0829_Identify(uint32 param)
{
#if 1
		uint8 reg[2]	= {0x02, 0x02};
		//uint8 value[2]  = {0x0c, 0x0c};	//0828
		uint8 value[2]	= {0x9a, 0x9a}; //0829
		uint8 ret		= 0;
		uint32 i;
		uint8 err_cnt = 0;
		uint32 nLoop = 1000;
	
	//	SP0829_DELAY_MS(10);
	
		for(i = 0; i<2; )
		{
			nLoop = 1000;
			//SP0829_WriteReg(0xfd,0x00);  
			ret = SP0829_ReadReg(reg[i]);	 
			
			SCI_TRACE_LOW( "SP0829 read 0x%x ret is %X",reg[i], ret );
			
			if( ret != value[i])
			{
				err_cnt++;
				if(err_cnt>3)
				{
					SCI_TRACE_LOW( "SP0829 Fail" );
					return 0xFF;
				}
				else
				{
					//Masked by frank.yang,SCI_Sleep() will cause a  Assert when called in boot precedure
					//SCI_Sleep(10);
					while(nLoop--);
					continue;
				}
			}
			err_cnt = 0;
			i++;
		}
	
		 SCI_TRACE_LOW( "SP0829 succ" );
#endif
	
	return (uint32)SCI_SUCCESS;
	
}


LOCAL SENSOR_TRIM_T s_SP0829_Pclk_Tab[]=
{	
	// COMMON INIT
	{0, 0, 0, 0,  6},
	
	// YUV422 PREVIEW 1	
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	{0, 0, 0, 0,  6},
	
	// YUV422 PREVIEW 2 
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0},
	{0, 0, 0, 0,  0}
};

LOCAL uint32 SP0829_GetPclkTab(uint32 param)
{
    return (uint32)s_SP0829_Pclk_Tab;
}


__align(4) const SENSOR_REG_T SP0829_brightness_tab[][2]=
{
       {		
		{0x78, 0xd0}, {0xff,0xff},
	},

	{
		{0x78, 0xe0},	{0xff,0xff},
	},

	{
		{0x78, 0xf0},	{0xff,0xff},
	},

	{
		{0x78, 0x00},	{0xff,0xff},
	},

	{
		{0x78, 0x20},	{0xff,0xff},
	},

	{
		{0x78, 0x30},	{0xff,0xff},
	},

	{
		{0x78, 0x40},	{0xff,0xff},
	},
};

LOCAL uint32 Set_SP0829_Brightness(uint32 level)
{
 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0829_brightness_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    SP0829_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0829_brightness: level = %d", level);
    
    return 0;
}



__align(4) const SENSOR_REG_T SP0829_contrast_tab[][2]=
{
	{
		{0x5c,0x70}, 	{0xff,0xff},
	},

	{
		{0x5c,0x80}, 	{0xff,0xff}, 
	},

	{
		{0x5c,0x90}, 	{0xff,0xff}, 
	},

	{
		{0x5c,0xa0}, 	{0xff,0xff},
	},

	{
		{0x5c,0xb0}, 	{0xff,0xff}, 
	},

	{
		{0x5c,0xc0}, 	{0xff,0xff}, 
	},

	{
		{0x5c,0xd0}, 	{0xff,0xff},
	},                   
};

LOCAL uint32 Set_SP0829_Contrast(uint32 level)
{
 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0829_contrast_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    SP0829_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0829_contrast: level = %d", level);
    
    return 0;
}


__align(4) const SENSOR_REG_T SP0829_image_effect_tab[][4]=	
{
    // effect normal
    {
		{0x62,0x00}, {0x63,0x80}, {0x64,0x80}, {0xff,0xff}
    },
    //effect BLACKWHITE
    {
		{0x62,0x08}, {0x63,0x80}, {0x64,0x80}, {0xff,0xff}
    },
    // effect RED pink
    {
		{0x62,0x04}, {0x63,0xb0}, {0x64,0x90}, {0xff,0xff}
    },
    // effect GREEN
    {
 		{0x62,0x04}, {0x63,0x50}, {0x64,0x50}, {0xff,0xff}
    },
    // effect  BLUE
    {
   		{0x62,0x04}, {0x63,0x20}, {0x64,0xf0}, {0xff,0xff}
    },
    // effect  YELLOW
    {
		{0x62,0x04}, {0x63,0xb0}, {0x64,0x00}, {0xff,0xff}
    },  
    // effect NEGATIVE
    {	     
   		{0x62,0x01}, {0x63,0x80}, {0x64,0x80}, {0xff,0xff}
    },    
    //effect ANTIQUE
    {
   		{0x62,0x04}, {0x63,0xc0}, {0x64,0x20}, {0xff,0xff}
    },
};

LOCAL uint32 Set_SP0829_Image_Effect(uint32 effect_type)
{
   
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0829_image_effect_tab[effect_type];

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    SP0829_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0829_image_effect: effect_type = %d", effect_type);
    
    return 0;
}



__align(4) const SENSOR_REG_T SP0829_ev_tab[][9]=
{   
    {{0x97, 0x78-0x18},{0x98, 0x88-0x18},{0x99, 0x74-0x18},{0x9a, 0x84-0x18},{0x9e, 0x84-0x18},{0x9f, 0x7c-0x18},{0xa0, 0x80-0x18},{0xa1, 0x78-0x18},{0xff, 0xff}},
    {{0x97, 0x78-0x10},{0x98, 0x88-0x10},{0x99, 0x74-0x10},{0x9a, 0x84-0x10},{0x9e, 0x84-0x10},{0x9f, 0x7c-0x10},{0xa0, 0x80-0x10},{0xa1, 0x78-0x10},{0xff, 0xff}},
    {{0x97, 0x78-0x08},{0x98, 0x88-0x08},{0x99, 0x74-0x08},{0x9a, 0x84-0x08},{0x9e, 0x84-0x08},{0x9f, 0x7c-0x08},{0xa0, 0x80-0x08},{0xa1, 0x78-0x08},{0xff, 0xff}},
    {{0x97, 0x78-0x00},{0x98, 0x88-0x00},{0x99, 0x74-0x00},{0x9a, 0x84-0x00},{0x9e, 0x84-0x00},{0x9f, 0x7c-0x00},{0xa0, 0x80-0x00},{0xa1, 0x78-0x00},{0xff, 0xff}},
    {{0x97, 0x78+0x08},{0x98, 0x88+0x08},{0x99, 0x74+0x08},{0x9a, 0x84+0x08},{0x9e, 0x84+0x08},{0x9f, 0x7c+0x08},{0xa0, 0x80+0x08},{0xa1, 0x78+0x08},{0xff, 0xff}},
    {{0x97, 0x78+0x10},{0x98, 0x88+0x10},{0x99, 0x74+0x10},{0x9a, 0x84+0x10},{0x9e, 0x84+0x10},{0x9f, 0x7c+0x10},{0xa0, 0x80+0x10},{0xa1, 0x78+0x10},{0xff, 0xff}},
    {{0x97, 0x78+0x18},{0x98, 0x88+0x18},{0x99, 0x74+0x18},{0x9a, 0x84+0x18},{0x9e, 0x84+0x18},{0x9f, 0x7c+0x18},{0xa0, 0x80+0x18},{0xa1, 0x78+0x18},{0xff, 0xff}},  
};

LOCAL uint32 Set_SP0829_Ev(uint32 level)
{
  
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0829_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    SCI_ASSERT(level < 7);
 
    SP0829_Write_Group_Regs(sensor_reg_ptr );

    SCI_TRACE_LOW("set_SP0829_ev: level = %d", level);

    return 0;
}

/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/
LOCAL uint32 Set_SP0829_Anti_Flicker(uint32 mode)
{ 
	switch(mode)
	{
	case DCAMERA_FLICKER_50HZ:
	
		bl_GC_50Hz_SP0829 = TRUE;
		SCI_TRACE_LOW( " set_SP0829_anti_flicker  50hz \r\n" );

         break;       
	case DCAMERA_FLICKER_60HZ:
	
	    bl_GC_50Hz_SP0829 = FALSE;
	       SCI_TRACE_LOW( " set_SP0829_anti_flicker  60hz \r\n" );
			 
	break;

	    default:
	    break;
    }
      
    SCI_TRACE_LOW("set_SP0829_anti_flicker-mode=%d",mode);
    OS_TickDelay(100); 
	
    return 0;
}
      


LOCAL uint32 Set_SP0829_Preview_Mode(uint32 preview_mode)
{
	switch (preview_mode)
	{
		case DCAMERA_ENVIRONMENT_NORMAL: 
		{
			if( bl_GC_50Hz_SP0829 == TRUE)
			{
		SCI_TRACE_LOW("normal mode 50hz\r\n");
		#ifdef 	SP0829_OUTPUT_MODE_SPI
		//capture preview daylight 50HZ 24M 8Ƶ for 1bit 15-8fps 
	        SP0829_WriteReg(0x05,0x00);			 
	        SP0829_WriteReg(0x06,0x00);			 
	        SP0829_WriteReg(0x09,0x00);			 
	        SP0829_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0829_WriteReg(0x9b,0x32);//base			 
	        SP0829_WriteReg(0x9c,0x00);			 
	                        		  	   
	        SP0829_WriteReg(0xa2,0x58);//exp			 
	        SP0829_WriteReg(0xa3,0x02);			 
	        SP0829_WriteReg(0xa4,0x32);			 
	        SP0829_WriteReg(0xa5,0x00);			 
	        SP0829_WriteReg(0xa8,0x32);			 
	        SP0829_WriteReg(0xa9,0x00);			 
	        SP0829_WriteReg(0xaa,0x01);			 
	        SP0829_WriteReg(0xab,0x00);
	        
		SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)
           //capture preview daylight 50HZ 24M 4Ƶ for 2bit 20-8fps  
           
                SP0829_WriteReg(0x05,0x00);		
                SP0829_WriteReg(0x06,0x00);		
                SP0829_WriteReg(0x09,0x00);		
                SP0829_WriteReg(0x0a,0xc0);       		
                 				        
                SP0829_WriteReg(0x9b,0x43);//base	
                SP0829_WriteReg(0x9c,0x00);		
                                		                
                SP0829_WriteReg(0xa2,0x24);//exp	
                SP0829_WriteReg(0xa3,0x03);		
                SP0829_WriteReg(0xa4,0x43);		
                SP0829_WriteReg(0xa5,0x00);		
                SP0829_WriteReg(0xa8,0x43);		
                SP0829_WriteReg(0xa9,0x00);		
                SP0829_WriteReg(0xaa,0x01);		
	        SP0829_WriteReg(0xab,0x00);   
	        
	        SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);
	        
	                
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)
          //capture preview daylight 50HZ 24M 2Ƶ for 4bit 20-8fps 

                 SP0829_WriteReg(0x05,0x00);		
                 SP0829_WriteReg(0x06,0x00);		
                 SP0829_WriteReg(0x09,0x02);		
                 SP0829_WriteReg(0x0a,0x80);       	
                  				            
                 SP0829_WriteReg(0x9b,0x43);//base	
                 SP0829_WriteReg(0x9c,0x00);		
                                 		            
                 SP0829_WriteReg(0xa2,0x24);//exp	
                 SP0829_WriteReg(0xa3,0x03);		
                 SP0829_WriteReg(0xa4,0x43);		
                 SP0829_WriteReg(0xa5,0x00);		
                 SP0829_WriteReg(0xa8,0x43);		
                 SP0829_WriteReg(0xa9,0x00);		
                 SP0829_WriteReg(0xaa,0x01);		
                 SP0829_WriteReg(0xab,0x00);    
                 
                 SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		 SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);
                              
#endif
		       }
		       else
		       {
		       	SCI_TRACE_LOW("normal mode 60hz\r\n");
		       	#ifdef 	SP0829_OUTPUT_MODE_SPI                          					
		//capture preview daylight 60HZ 24M 8Ƶ for 1bit 15-8fps      		
	        SP0829_WriteReg(0x05,0x00);				
	        SP0829_WriteReg(0x06,0x00);				
	        SP0829_WriteReg(0x09,0x00);				
	        SP0829_WriteReg(0x0a,0x40);  				
	         				        		
	        SP0829_WriteReg(0x9b,0x27);//base			
	        SP0829_WriteReg(0x9c,0x00);				
	                        		  			
	        SP0829_WriteReg(0xa2,0x49);//exp			
	        SP0829_WriteReg(0xa3,0x02);				
	        SP0829_WriteReg(0xa4,0x27);				
	        SP0829_WriteReg(0xa5,0x00);				
	        SP0829_WriteReg(0xa8,0x27);				
	        SP0829_WriteReg(0xa9,0x00);				
	        SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);             		
		
		SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);	
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)  
              //capture preview daylight 60HZ 24M 4Ƶ for 2bit 20-8fps     		
                                                        		
                SP0829_WriteReg(0x05,0x00);				
                SP0829_WriteReg(0x06,0x00);				
                SP0829_WriteReg(0x09,0x00);				
                SP0829_WriteReg(0x0a,0xbe);       			
                 				        		
                SP0829_WriteReg(0x9b,0x38);//base			
                SP0829_WriteReg(0x9c,0x00);				
                                		        		
                SP0829_WriteReg(0xa2,0x48);//exp			
                SP0829_WriteReg(0xa3,0x03);				
                SP0829_WriteReg(0xa4,0x38);				
                SP0829_WriteReg(0xa5,0x00);				
                SP0829_WriteReg(0xa8,0x38);				
                SP0829_WriteReg(0xa9,0x00);				
                SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);     
	        
	        SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);        		
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)  
             //capture preview daylight 60HZ 24M 2Ƶ for 4bit 20-8fps    		
                                                        		
                 SP0829_WriteReg(0x05,0x00);				
                 SP0829_WriteReg(0x06,0x00);				
                 SP0829_WriteReg(0x09,0x02);				
                 SP0829_WriteReg(0x0a,0x7d);       			
                  				        		
                 SP0829_WriteReg(0x9b,0x38);//base			
                 SP0829_WriteReg(0x9c,0x00);				
                                 		        		
                 SP0829_WriteReg(0xa2,0x48);//exp			
                 SP0829_WriteReg(0xa3,0x03);				
                 SP0829_WriteReg(0xa4,0x38);				
                 SP0829_WriteReg(0xa5,0x00);				
                 SP0829_WriteReg(0xa8,0x38);				
                 SP0829_WriteReg(0xa9,0x00);				
                 SP0829_WriteReg(0xaa,0x01);				
                 SP0829_WriteReg(0xab,0x00);    
                 
                 SP0829_WriteReg(0xa6,SP0829_DAYLIGHT_MAXGAIN);	
		 SP0829_WriteReg(0xac,SP0829_DAYLIGHT_MAXGAIN);        
#endif
		       	
		       	}     
			break;
		}
		case DCAMERA_ENVIRONMENT_NIGHT:
		{
			if( bl_GC_50Hz_SP0829 == TRUE)
			{
		SCI_TRACE_LOW("night mode 50hz\r\n");
		#ifdef 	SP0829_OUTPUT_MODE_SPI
		//capture preview night 50HZ 24M 8Ƶ for 1bit 15-5fps 
	        SP0829_WriteReg(0x05,0x00);			 
	        SP0829_WriteReg(0x06,0x00);			 
	        SP0829_WriteReg(0x09,0x00);			 
	        SP0829_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0829_WriteReg(0x9b,0x32);//base			 
	        SP0829_WriteReg(0x9c,0x00);			 
	                        		  	   
	        SP0829_WriteReg(0xa2,0xe8);//exp			 
	        SP0829_WriteReg(0xa3,0x03);			 
	        SP0829_WriteReg(0xa4,0x32);			 
	        SP0829_WriteReg(0xa5,0x00);			 
	        SP0829_WriteReg(0xa8,0x32);			 
	        SP0829_WriteReg(0xa9,0x00);			 
	        SP0829_WriteReg(0xaa,0x01);			 
	        SP0829_WriteReg(0xab,0x00);
	        
		SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN);	
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)
           //capture preview night 50HZ 24M 4Ƶ for 2bit 20-5fps  
           
                SP0829_WriteReg(0x05,0x00);		
                SP0829_WriteReg(0x06,0x00);		
                SP0829_WriteReg(0x09,0x00);		
                SP0829_WriteReg(0x0a,0xc0);       		
                 				        
                SP0829_WriteReg(0x9b,0x43);//base	
                SP0829_WriteReg(0x9c,0x00);		
                                		                
                SP0829_WriteReg(0xa2,0x3c);//exp	
                SP0829_WriteReg(0xa3,0x05);		
                SP0829_WriteReg(0xa4,0x43);		
                SP0829_WriteReg(0xa5,0x00);		
                SP0829_WriteReg(0xa8,0x43);		
                SP0829_WriteReg(0xa9,0x00);		
                SP0829_WriteReg(0xaa,0x01);		
	        SP0829_WriteReg(0xab,0x00);   
	        
	        SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN);
	        
	                
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)
          //capture preview night 50HZ 24M 2Ƶ for 4bit 20-5fps 

                 SP0829_WriteReg(0x05,0x00);		
                 SP0829_WriteReg(0x06,0x00);		
                 SP0829_WriteReg(0x09,0x02);		
                 SP0829_WriteReg(0x0a,0x80);       	
                  				            
                 SP0829_WriteReg(0x9b,0x43);//base	
                 SP0829_WriteReg(0x9c,0x00);		
                                 		            
                 SP0829_WriteReg(0xa2,0x3c);//exp	
                 SP0829_WriteReg(0xa3,0x05);		
                 SP0829_WriteReg(0xa4,0x43);		
                 SP0829_WriteReg(0xa5,0x00);		
                 SP0829_WriteReg(0xa8,0x43);		
                 SP0829_WriteReg(0xa9,0x00);		
                 SP0829_WriteReg(0xaa,0x01);		
                 SP0829_WriteReg(0xab,0x00);    
                 
                 SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		 SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN);
                              
#endif
		       }
		       else
		       {
		       	SCI_TRACE_LOW("night mode 60hz\r\n");
		       		#ifdef 	SP0829_OUTPUT_MODE_SPI                          					
		//capture preview night 60HZ 24M 8Ƶ for 1bit 15-5fps     		
	        SP0829_WriteReg(0x05,0x00);				
	        SP0829_WriteReg(0x06,0x00);				
	        SP0829_WriteReg(0x09,0x00);				
	        SP0829_WriteReg(0x0a,0x40);  				
	         				        		
	        SP0829_WriteReg(0x9b,0x27);//base			
	        SP0829_WriteReg(0x9c,0x00);				
	                        		  			
	        SP0829_WriteReg(0xa2,0xa8);//exp			
	        SP0829_WriteReg(0xa3,0x03);				
	        SP0829_WriteReg(0xa4,0x27);				
	        SP0829_WriteReg(0xa5,0x00);				
	        SP0829_WriteReg(0xa8,0x27);				
	        SP0829_WriteReg(0xa9,0x00);				
	        SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);   
	                  		
		SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN);	
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)  
              //capture preview night 60HZ 24M 4Ƶ for 2bit 20-5fps   	
                                                        		
                SP0829_WriteReg(0x05,0x00);				
                SP0829_WriteReg(0x06,0x00);				
                SP0829_WriteReg(0x09,0x00);				
                SP0829_WriteReg(0x0a,0xbe);       			
                 				        		
                SP0829_WriteReg(0x9b,0x38);//base			
                SP0829_WriteReg(0x9c,0x00);				
                                		        		
                SP0829_WriteReg(0xa2,0x40);//exp			
                SP0829_WriteReg(0xa3,0x05);				
                SP0829_WriteReg(0xa4,0x38);				
                SP0829_WriteReg(0xa5,0x00);				
                SP0829_WriteReg(0xa8,0x38);				
                SP0829_WriteReg(0xa9,0x00);				
                SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);     
	        
	        SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN);        		
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)  
             //capture preview night 60HZ 24M 2Ƶ for 4bit 20-5fps   	
                                                        		
                 SP0829_WriteReg(0x05,0x00);				
                 SP0829_WriteReg(0x06,0x00);				
                 SP0829_WriteReg(0x09,0x02);				
                 SP0829_WriteReg(0x0a,0x7d);       			
                  				        		
                 SP0829_WriteReg(0x9b,0x38);//base			
                 SP0829_WriteReg(0x9c,0x00);				
                                 		        		
                 SP0829_WriteReg(0xa2,0x40);//exp			
                 SP0829_WriteReg(0xa3,0x05);				
                 SP0829_WriteReg(0xa4,0x38);				
                 SP0829_WriteReg(0xa5,0x00);				
                 SP0829_WriteReg(0xa8,0x38);				
                 SP0829_WriteReg(0xa9,0x00);				
                 SP0829_WriteReg(0xaa,0x01);				
                 SP0829_WriteReg(0xab,0x00);    
                 
                 SP0829_WriteReg(0xa6,SP0829_NIGHT_MAXGAIN);	
		SP0829_WriteReg(0xac,SP0829_NIGHT_MAXGAIN); 
                         
#endif
		       	
		       	}     
		       	break;
	   }
		default:
		{
			break;
		}
		
	}       
      SCI_TRACE_LOW("set_SP0829_preview_mode: level = %d", preview_mode);
      return 0;
}


__align(4) const SENSOR_REG_T SP0829_awb_tab[][6]=
{
    //AUTO
    {
        {0xc5, 0xc4},  //red  
        {0xc6, 0x9f},  //blue 
        {0xe7, 0x03},
        {0xe7, 0x00},
        {0x32, 0x0d},   // Enable AWB
        {0xff, 0xff}
    },    
    //INCANDESCENCE:
    {   {0x32, 0x05}, // Disable AWB
        {0xc5, 0xa8},  //red
        {0xc6, 0xae},  //blue
        {0xff, 0xff} 
    },
    //U30
    {     {0x32, 0x05},
          {0xc5, 0x85}, 
	  {0xc6, 0xea},
	  {0xff, 0xff} 
    },  
    //CWF  //
    {     {0x32, 0x05},
          {0xc5, 0xa8},   
          {0xc6, 0xb7},
          {0xff, 0xff} 
    },    
    //FLUORESCENT:
    {     {0x32, 0x05},
          {0xc5, 0x9c},    
          {0xc6, 0xaf},
          {0xff, 0xff} 
    },
    //SUN:
    {     {0x32, 0x05},
          {0xc5, 0xc5},   
          {0xc6, 0x8a},
          {0xff, 0xff} 
    },
    //CLOUD:
    {     {0x32, 0x05},
          {0xc5, 0xe5},    
          {0xc6, 0x80},
          {0xff, 0xff} 
    },
};

LOCAL uint32 Set_SP0829_AWB(uint32 mode)
{
	
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0829_awb_tab[mode];

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	SP0829_Write_Group_Regs(sensor_reg_ptr);
	
	SCI_TRACE_LOW("set_SP0829_awb_mode: mode = %d", mode);
	
	return 0;
}

LOCAL uint32 SP0829_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable)
{
	uint16 ae_value = 0 , awb_value=0;

	awb_value = SP0829_ReadReg(0x32);

	if(0x01==ae_enable)
	{
		ae_value |= 0x01;
	}
	else if(0x00==ae_enable)
	{
		ae_value &=0xfe;
	}

	if(0x01==awb_enable)
	{
		awb_value |=0x08;
	}
	else if(0x00==awb_enable)
	{
		awb_value &= 0xf7;
	}	
	
	SP0829_WriteReg(0x32, ae_value);
	SP0829_WriteReg(0x32, awb_value);

       SCI_TRACE_LOW("SENSOR: set_ae_awb_enable: ae_enable = %d  awb_enable = %d", ae_enable ,awb_enable);
	return 0;
}


LOCAL uint32 SP0829_Before_Snapshot(uint32 para)
{
	uint8 reg_val = 0;
	uint16 exposal_time=0x00;

	SP0829_AE_AWB_Enable(0x32,0x00);   // close aec awb

	reg_val = SP0829_ReadReg(0x04);
	exposal_time=reg_val&0x00ff;
	reg_val = SP0829_ReadReg(0x03);
	exposal_time|=((reg_val&0x00ff)<<0x08);

	if(exposal_time<1)
	{
		exposal_time=1;
	}

	reg_val=exposal_time&0x00ff;
	SP0829_WriteReg(0x04, reg_val);
	reg_val=(exposal_time&0xff00)>>8;
	SP0829_WriteReg(0x03, reg_val);

	SP0829_Delayms(400);	
	
	SCI_TRACE_LOW("SENSOR_SP0829: Before Snapshot");

	return 0;
}



LOCAL uint32 SP0829_After_Snapshot(uint32 para)
{
	SP0829_AE_AWB_Enable(0x32,0x0d);   // Open aec awb
	
	SP0829_Delayms(400);
	
	SCI_TRACE_LOW("SENSOR_SP0829: After Snapshot");
	//SP0829_Power_On(SCI_FALSE);

	return 0;
}


 
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/   
LOCAL uint32 Set_SP0829_Video_Mode(uint32 mode)
{
    
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

    if(1==mode ||2==mode )
    {
    	if(bl_GC_50Hz_SP0829 == TRUE)  // is 50hz in video
    	{
#ifdef 	SP0829_OUTPUT_MODE_SPI
		//AE 24M 8Ƶ for 1bit fix 14fps
	        SP0829_WriteReg(0x05,0x00);			 
	        SP0829_WriteReg(0x06,0x00);			 
	        SP0829_WriteReg(0x09,0x00);			 
	        SP0829_WriteReg(0x0a,0x2c);  			 
	         				   
	        SP0829_WriteReg(0x9b,0x32);//base			 
	        SP0829_WriteReg(0x9c,0x00);			 
	                        		  	   
	        SP0829_WriteReg(0xa2,0x5e);//exp			 
	        SP0829_WriteReg(0xa3,0x01);			 
	        SP0829_WriteReg(0xa4,0x32);			 
	        SP0829_WriteReg(0xa5,0x00);			 
	        SP0829_WriteReg(0xa8,0x32);			 
	        SP0829_WriteReg(0xa9,0x00);			 
	        SP0829_WriteReg(0xaa,0x01);			 
	        SP0829_WriteReg(0xab,0x00);
			
		SP0829_WriteReg(0xa6,0xa0);	
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)
  //AE 24M 4Ƶ for 2bit fix 14fps
                SP0829_WriteReg(0x05,0x00);		
                SP0829_WriteReg(0x06,0x00);		
                SP0829_WriteReg(0x09,0x01);		
                SP0829_WriteReg(0x0a,0x7e);       		
                 				        
                SP0829_WriteReg(0x9b,0x2f);//base	
                SP0829_WriteReg(0x9c,0x00);		
                                		                
                SP0829_WriteReg(0xa2,0x49);//exp	
                SP0829_WriteReg(0xa3,0x01);		
                SP0829_WriteReg(0xa4,0x2f);		
                SP0829_WriteReg(0xa5,0x00);		
                SP0829_WriteReg(0xa8,0x2f);		
                SP0829_WriteReg(0xa9,0x00);		
                SP0829_WriteReg(0xaa,0x01);		
	        SP0829_WriteReg(0xab,0x00);   
	        
	         SP0829_WriteReg(0xa6,0xa0); 
	        
	        //AE 24M 2Ƶ for 4bit fix 14fps          
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)

                 SP0829_WriteReg(0x05,0x00);		
                 SP0829_WriteReg(0x06,0x00);		
                 SP0829_WriteReg(0x09,0x03);		
                 SP0829_WriteReg(0x0a,0xfd);       	
                  				            
                 SP0829_WriteReg(0x9b,0x2f);//base	
                 SP0829_WriteReg(0x9c,0x00);		
                                 		            
                 SP0829_WriteReg(0xa2,0x49);//exp	
                 SP0829_WriteReg(0xa3,0x01);		
                 SP0829_WriteReg(0xa4,0x2f);		
                 SP0829_WriteReg(0xa5,0x00);		
                 SP0829_WriteReg(0xa8,0x2f);		
                 SP0829_WriteReg(0xa9,0x00);		
                 SP0829_WriteReg(0xaa,0x01);		
                 SP0829_WriteReg(0xab,0x00);      
                 
                  SP0829_WriteReg(0xa6,0xa0);            
#endif

	}
	else  // is 60hz in video mode
	{
#ifdef 	SP0829_OUTPUT_MODE_SPI                          					
		//AE 24M 8Ƶ for 1bit fix 14fps           		
	        SP0829_WriteReg(0x05,0x00);				
	        SP0829_WriteReg(0x06,0x00);				
	        SP0829_WriteReg(0x09,0x00);				
	        SP0829_WriteReg(0x0a,0x40);  				
	         				        		
	        SP0829_WriteReg(0x9b,0x27);//base			
	        SP0829_WriteReg(0x9c,0x00);				
	                        		  			
	        SP0829_WriteReg(0xa2,0x38);//exp			
	        SP0829_WriteReg(0xa3,0x01);				
	        SP0829_WriteReg(0xa4,0x27);				
	        SP0829_WriteReg(0xa5,0x00);				
	        SP0829_WriteReg(0xa8,0x27);				
	        SP0829_WriteReg(0xa9,0x00);				
	        SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);    
			
		 SP0829_WriteReg(0xa6,0xa0); 	
  #elif defined(SP0829_OUTPUT_MODE_CCIR656_2BIT)  
  //AE 24M 4Ƶ for 2bit fix 14fps       		
                                                        		
                SP0829_WriteReg(0x05,0x00);				
                SP0829_WriteReg(0x06,0x00);				
                SP0829_WriteReg(0x09,0x01);				
                SP0829_WriteReg(0x0a,0x81);       			
                 				        		
                SP0829_WriteReg(0x9b,0x27);//base			
                SP0829_WriteReg(0x9c,0x00);				
                                		        		
                SP0829_WriteReg(0xa2,0x38);//exp			
                SP0829_WriteReg(0xa3,0x01);				
                SP0829_WriteReg(0xa4,0x27);				
                SP0829_WriteReg(0xa5,0x00);				
                SP0829_WriteReg(0xa8,0x27);				
                SP0829_WriteReg(0xa9,0x00);				
                SP0829_WriteReg(0xaa,0x01);				
	        SP0829_WriteReg(0xab,0x00);     
	        
	         SP0829_WriteReg(0xa6,0xa0);         		
#elif defined(SP0829_OUTPUT_MODE_CCIR656_4BIT)  
//AE 24M 2Ƶ for 4bit fix 14fps          		
                                                        		
                 SP0829_WriteReg(0x05,0x00);				
                 SP0829_WriteReg(0x06,0x00);				
                 SP0829_WriteReg(0x09,0x04);				
                 SP0829_WriteReg(0x0a,0x02);       			
                  				        		
                 SP0829_WriteReg(0x9b,0x27);//base			
                 SP0829_WriteReg(0x9c,0x00);				
                                 		        		
                 SP0829_WriteReg(0xa2,0x38);//exp			
                 SP0829_WriteReg(0xa3,0x01);				
                 SP0829_WriteReg(0xa4,0x27);				
                 SP0829_WriteReg(0xa5,0x00);				
                 SP0829_WriteReg(0xa8,0x27);				
                 SP0829_WriteReg(0xa9,0x00);				
                 SP0829_WriteReg(0xaa,0x01);				
                 SP0829_WriteReg(0xab,0x00);    
                 
                  SP0829_WriteReg(0xa6,0xa0);         
#endif				 
	}
    SCI_TRACE_LOW("set_SP0829_video_mode=%d",mode);
    }	
	
    return 0;
}

#ifdef SUB_CAMERA_SUPPORT
void Init_Main_Camera_For_Sub(SENSOR_OUTPUT_MODE_E mode)
{
	if(SENSOR_OUTPUT_MODE_SPI == mode)
	{
		SP0829_WriteReg(0x1c,0x00);
		SP0829_WriteReg(0x2e,0x80); //	for sp0827						  
		SP0829_WriteReg(0x2c,0x0b);
		SP0829_WriteReg(0x31,0x00);
		SP0829_WriteReg(0x30,0x06); //PCLK ƵISP 8Ƶ					   
	}
	else
	{
		SP0829_WriteReg(0x2e,0x91);//	for sp0827
		SP0829_WriteReg(0x2c,0x08);
		SP0829_WriteReg(0x1c,0x00);
		SP0829_WriteReg(0x1b,0x01);
		SP0829_WriteReg(0x30,0x04); //PCLK ƵISP 4Ƶ					  
		SP0829_WriteReg(0x31,0x00); 
	}

	SP0829_WriteReg(0x0e,0xf7);
	SP0829_WriteReg(0x35,0x40);
	SP0829_WriteReg(0x32,0x00);		
	//AWB 	   
	SP0829_WriteReg(0xc5,0xc4);
	SP0829_WriteReg(0xc6,0x9f);
	SP0829_WriteReg(0xE7,0x03);
	SP0829_WriteReg(0xE7,0x00);
	SP0829_WriteReg(0xbe,0x20);
	SP0829_WriteReg(0xb3,0x7e);
	SP0829_WriteReg(0xb5,0x10);
	SP0829_WriteReg(0xd4,0xff);
	SP0829_WriteReg(0xba,0x2d);
	SP0829_WriteReg(0xd7,0x90);
	SP0829_WriteReg(0xd8,0xa0);
	SP0829_WriteReg(0xd9,0xff);
	SP0829_WriteReg(0xdd,0xa6);
	SP0829_WriteReg(0xe1,0xc7);
	SP0829_WriteReg(0xe3,0xd0);
	SP0829_WriteReg(0xd5,0xe0);    

	//smooth	   
	SP0829_WriteReg(0x42,0x54);//03 02
	SP0829_WriteReg(0x46,0x0a);

	//sharpen    
	SP0829_WriteReg(0x50,0x40);
	SP0829_WriteReg(0x51,0x10);
	SP0829_WriteReg(0x52,0xe0);
	SP0829_WriteReg(0x55,0x10);
						 
	SP0829_WriteReg(0x56,0x30);
	SP0829_WriteReg(0x57,0x20);
	//cm		  
	SP0829_WriteReg(0x79,0xb1);
	SP0829_WriteReg(0x7a,0xcf);
	SP0829_WriteReg(0x7b,0x00);
	SP0829_WriteReg(0x7c,0x19);
	SP0829_WriteReg(0x7d,0x70);
	SP0829_WriteReg(0x7e,0xf7);
	SP0829_WriteReg(0x7f,0x18);
	SP0829_WriteReg(0x80,0xc1);
	SP0829_WriteReg(0x81,0xa6);
	SP0829_WriteReg(0x82,0x0c);
	SP0829_WriteReg(0x83,0x30);
	SP0829_WriteReg(0x84,0x0c);
	//sat
	SP0829_WriteReg(0x4f,0x60);
	SP0829_WriteReg(0x58,0xff);
	SP0829_WriteReg(0x59,0xff);
	//heq
	SP0829_WriteReg(0x5b,0x80);
	SP0829_WriteReg(0x5c,0xa0);
	//auto lum lowlight
	SP0829_WriteReg(0x86,0x30);
	SP0829_WriteReg(0x87,0x1f);
	SP0829_WriteReg(0x88,0x30);
        SP0829_WriteReg(0x89,0x45);
	//gamma	 
	SP0829_WriteReg(0x65,0x0 );
	SP0829_WriteReg(0x66,0xa );
	SP0829_WriteReg(0x67,0x13);
	SP0829_WriteReg(0x68,0x25);
	SP0829_WriteReg(0x69,0x43);
	SP0829_WriteReg(0x6a,0x5d);
	SP0829_WriteReg(0x6b,0x74);
	SP0829_WriteReg(0x6c,0x88);
	SP0829_WriteReg(0x6d,0x9a);
	SP0829_WriteReg(0x6e,0xa9);
	SP0829_WriteReg(0x6f,0xb5);
	SP0829_WriteReg(0x70,0xc0);
	SP0829_WriteReg(0x71,0xca);
	SP0829_WriteReg(0x72,0xd4);
	SP0829_WriteReg(0x73,0xdd);
	SP0829_WriteReg(0x74,0xe6);
	SP0829_WriteReg(0x75,0xef);
	SP0829_WriteReg(0x76,0xf7);
	SP0829_WriteReg(0x77,0xff);  
 
	SP0829_WriteReg(0x33,0x0f); 
	SP0829_WriteReg(0x34,0x06);  
	SP0829_WriteReg(0x32,0x08); 
}

uint32 Set_Main_Camera_Brightness(uint32 level)
{
	return Set_SP0829_Brightness(level);
}

uint32 Set_Main_Camera_Contrast(uint32 level)
{
	return Set_SP0829_Contrast(level);
}

uint32 Set_Main_Camera_Image_Effect(uint32 effect_type)
{
	return Set_SP0829_Image_Effect(effect_type);
}

__align(4) const SENSOR_REG_T sub_cam_awb_tab[][6]=
{
    //AUTO
    {
        {0xc5, 0xc4},  //red  
        {0xc6, 0x9f},  //blue 
        {0xe7, 0x03},
        {0xe7, 0x00},
        {0x32, 0x08},   // Enable AWB
        {0xff, 0xff}
    },    
    //INCANDESCENCE:
    {   {0x32, 0x00}, // Disable AWB
        {0xc5, 0xa8},  //red
        {0xc6, 0xae},  //blue
        {0xff, 0xff} 
    },
    //U30
    {     {0x32, 0x00},
          {0xc5, 0x85}, 
	  {0xc6, 0xea},
	  {0xff, 0xff} 
    },  
    //CWF  //
    {     {0x32, 0x00},
          {0xc5, 0xa8},   
          {0xc6, 0xb7},
          {0xff, 0xff} 
    },    
    //FLUORESCENT:
    {     {0x32, 0x00},
          {0xc5, 0x9c},    
          {0xc6, 0xaf},
          {0xff, 0xff} 
    },
    //SUN:
    {     {0x32, 0x00},
          {0xc5, 0xc5},   
          {0xc6, 0x8a},
          {0xff, 0xff} 
    },
    //CLOUD:
    {     {0x32, 0x00},
          {0xc5, 0xe5},    
          {0xc6, 0x80},
          {0xff, 0xff} 
    },
};

uint32 Set_Main_Camera_AWB(uint32 mode)
{
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)sub_cam_awb_tab[mode];

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	SP0829_Write_Group_Regs(sensor_reg_ptr);
	
	SCI_TRACE_LOW("set_Main_Camera_awb_mode: mode = %d", mode);
	
	return 0;
}

#endif

/*lint -restore */
/***************************************
ATC function used by AT command for FAE
easy to check the effect of single reg value

BESURE WE HAVE REVISE PLATFORM TO SUPPORT THIS
use ate1 to show command
use at+sdatarxmd=A,B,C to write or read reg
A  1:write  0:read
B  0:select camera  other:reg address 
C  if(B==0)  0:main camera
                  1:sub camera
    if(B!=0)  value
NOTE THESE PARAMS ARE  DECIMAL

***************************************/
void ATC_WriteReg(uint8 add, uint8 val)
{
	if(0 == add)
		g_isMain = !val;
	else if(g_isMain)
		SP0829_WriteReg(add, val);
	else
		ATC27_WriteReg(add, val);
}
uint8 ATC_ReadReg(uint8 add)
{
	if(g_isMain)
		return SP0829_ReadReg(add);
	else
		return ATC27_ReadReg(add);
}
/**---------------------------------------------------------------------------------------------*
 **                         Compiler Flag                             					        *
 **---------------------------------------------------------------------------------------------*/
#ifdef   __cplusplus
    }
#endif

#endif
